<?php

    namespace App\Actions\Fortify;

    use App\Models\Team;
    use App\Models\User;
    use Illuminate\Support\Facades\DB;
    use Illuminate\Support\Facades\Hash;
    use Illuminate\Support\Facades\Validator;
    use Illuminate\Support\Str;
    use Laravel\Fortify\Contracts\CreatesNewUsers;
    use Laravel\Jetstream\Jetstream;


    class CreateNewUser implements CreatesNewUsers
    {
        use PasswordValidationRules;

        /**
         * Create a newly registered user.
         *
         * @param  array<string, string>  $input
         */
        public function create(array $input): User
        {
            Validator::make($input, [
                'name' => ['required', 'string', 'max:255'],
                'email' => ['required', 'string', 'email', 'max:255', 'unique:users'],
                'phone_number' => ['numeric', 'min:10'],
                'password' => $this->passwordRules(),
                'terms' => Jetstream::hasTermsAndPrivacyPolicyFeature() ? ['accepted', 'required'] : '',
            ])->validate();

            return DB::transaction(function () use ($input) {
                return tap(User::create([
                    'unique_id' => Str::uuid(),
                    'name' => $input['name'],
                    'company' => $input['company'],
                    'email' => $input['email'],
                    'phone_number' => $input['phone_number'],
                    'password' => Hash::make($input['password']),
                ]), function (User $user) {
                    $this->createTeam($user);
                    $this->createSuperadmin($user);
                });
            });
        }

        /**
         * Create a personal team for the user.
         */
        protected function createTeam(User $user): void
        {
            if ($user['company']) {
                $company_name = $user->company;
            } else {
                $company_name = explode(' ', $user->name, 2)[0]."'s Team";
            }

            $user->ownedTeams()->save(Team::forceCreate([
                'user_id' => $user->id,
                'name' => $company_name,
                'personal_team' => true,
            ]));
        }

        /**
         * Create the super Admin account for the first user.
         */
        protected function createSuperadmin(User $user): void
        {
            if ($user['id'] == '1') {
                DB::table('users')->where('id', $user['id'])->update([
                    'is_agent' => '1'
                ]);
            }
        }
    }
